@@ -3,7 +3,7 @@
 -- Module      :  Network.XmlRpc.Client
 -- Copyright   :  (c) Bjorn Bringert 2003
 -- License     :  BSD-style
--- 
+--
 -- Maintainer  :  bjorn@bringert.net
 -- Stability   :  experimental
 -- Portability :  non-portable (requires extensions and non-portable libraries)
@@ -19,7 +19,7 @@
 -- >
 -- > add :: String -> Int -> Int -> IO Int
 -- > add url = remote url "examples.add"
--- > 
+-- >
 -- > main = do
 -- >        let x = 4
 -- >            y = 7
@@ -48,9 +48,9 @@ import Network.Socket (withSocketsDo)
 import Network.HTTP
 import Network.Stream
 
-import Data.ByteString.Lazy.Char8 (ByteString, toChunks, fromChunks)
+import qualified Data.ByteString.Lazy.Char8 as BSL (ByteString, toChunks)
 import qualified Data.ByteString.UTF8 as U
-import qualified Data.ByteString as BS
+import qualified Data.ByteString.Char8 as BS
 
 -- | Gets the return value from a method response.
 --   Throws an exception if the response was a fault.
@@ -61,8 +61,8 @@ handleResponse (Fault code str) = fail ("Error " ++ show code ++ ": " ++ str)
 -- | Sends a method call to a server and returns the response.
 --   Throws an exception if the response was an error.
 doCall :: String -> MethodCall -> Err IO MethodResponse
-doCall url mc = 
-    do 
+doCall url mc =
+    do
     let req = renderCall mc
     --FIXME: remove
     --putStrLn req
@@ -72,7 +72,7 @@ doCall url mc =
     parseResponse resp
 
 -- | Low-level method calling function. Use this function if
---   you need to do custom conversions between XML-RPC types and 
+--   you need to do custom conversions between XML-RPC types and
 --   Haskell types.
 --   Throws an exception if the response was a fault.
 call :: String -- ^ URL for the XML-RPC server.
@@ -83,19 +83,19 @@ call url method args = doCall url (MethodCall method args) >>= handleResponse
 
 
 -- | Call a remote method.
-remote :: Remote a => 
+remote :: Remote a =>
 	  String -- ^ Server URL. May contain username and password on
 	         --   the format username:password\@ before the hostname.
        -> String -- ^ Remote method name.
-       -> a      -- ^ Any function 
-		 -- @(XmlRpcType t1, ..., XmlRpcType tn, XmlRpcType r) => 
+       -> a      -- ^ Any function
+		 -- @(XmlRpcType t1, ..., XmlRpcType tn, XmlRpcType r) =>
                  -- t1 -> ... -> tn -> IO r@
 remote u m = remote_ (\e -> "Error calling " ++ m ++ ": " ++ e) (call u m)
 
 class Remote a where
     remote_ :: (String -> String)        -- ^ Will be applied to all error
 					 --   messages.
-	    -> ([Value] -> Err IO Value) 
+	    -> ([Value] -> Err IO Value)
 	    -> a
 
 instance XmlRpcType a => Remote (IO a) where
@@ -121,7 +121,7 @@ handleE _ (Right v) = return v
 -- | Post some content to a uri, return the content of the response
 --   or an error.
 -- FIXME: should we really use fail?
-post :: String -> ByteString -> IO String
+post :: String -> BSL.ByteString -> IO String
 post url content = do
     uri <- maybeFail ("Bad URI: '" ++ url ++ "'") (parseURI url)
     let a = uriAuthority uri
@@ -132,10 +132,10 @@ post url content = do
 -- | Post some content to a uri, return the content of the response
 --   or an error.
 -- FIXME: should we really use fail?
-post_ :: URI -> URIAuth -> ByteString -> IO String
-post_ uri auth content = 
+post_ :: URI -> URIAuth -> BSL.ByteString -> IO String
+post_ uri auth content =
     do
-    eresp <- simpleHTTP (request uri auth (BS.concat . toChunks $ content))
+    eresp <- simpleHTTP (request uri auth (BS.concat . BSL.toChunks $ content))
     resp <- handleE (fail . show) eresp
     case rspCode resp of
 		      (2,0,0) -> return (U.toString (rspBody resp))
@@ -146,9 +146,9 @@ post_ uri auth content =
 
 -- | Create an XML-RPC compliant HTTP request.
 request :: URI -> URIAuth -> BS.ByteString -> Request BS.ByteString
-request uri auth content = Request{ rqURI = uri, 
-				    rqMethod = POST, 
-				    rqHeaders = headers, 
+request uri auth content = Request{ rqURI = uri,
+				    rqMethod = POST,
+				    rqHeaders = headers,
 				    rqBody = content }
     where
     -- the HTTP module adds a Host header based on the URI
@@ -160,19 +160,16 @@ request uri auth content = Request{ rqURI = uri,
                          in ( if null u then Nothing else Just u
                             , if null pw then Nothing else Just (tail pw))
 
--- | Creates an Authorization header using the Basic scheme, 
+-- | Creates an Authorization header using the Basic scheme,
 --   see RFC 2617 section 2.
 authHdr :: Maybe String -- ^ User name, if any
 	-> Maybe String -- ^ Password, if any
-	-> Maybe Header -- ^ If user name or password was given, returns 
+	-> Maybe Header -- ^ If user name or password was given, returns
 	                --   an Authorization header, otherwise 'Nothing'
 authHdr Nothing Nothing = Nothing
 authHdr u p = Just (Header HdrAuthorization ("Basic " ++ base64encode user_pass))
-	where user_pass = fromMaybe "" u ++ ":" ++ fromMaybe "" p
-	      base64encode = Base64.encode . stringToOctets
-	      -- FIXME: this probably only works right for latin-1 strings
-	      stringToOctets :: String -> [Word8]
-	      stringToOctets = map (fromIntegral . fromEnum)
+	where user_pass    = fromMaybe "" u ++ ":" ++ fromMaybe "" p
+	      base64encode = BS.unpack . Base64.encode . BS.pack
 
 --
 -- Utility functions
